/*******************************************************************************
 * Project name:
     Button R click
 * Copyright:
     (c) Mikroelektronika, 2015
 * Revision History:
     20150802:
       - initial release (DM)
       - mikroC PRO for ARM (AM)
 * Description:
     This program demonstrates usage of the Button R click. In this example light
     on the button switch (6.8mm in diameter and has a red LED backlight)
     is controlled by PWM as well as TFT display backlight.
     When pressed, it sends an interrupt signal to the target board microcontroller.

 * Test configuration:
     MCU:             STM32F107VC
                      http://www.st.com/st-web-ui/static/active/en/resource/technical/document/reference_manual/CD00171190.pdf
     Dev. Board:      EasyMx PRO v7 for STM32
                      http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
     Oscillator:      HS-PLL 72.0000 MHz
                      http://www.mikroe.com/store/components/displays/
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 * NOTES:
     - Place Button R click board at the mikroBUS socket 1 on the EasyMx Pro v7 board.
     - Turn on SW14.8 for buzzer
     - Put EasyTFT in its socket, turn on backlight using the SW11.3, and backlight PWM using SW11.4
*******************************************************************************/

#include "resources.h"

#define PWM_step 1.04427375

// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections

unsigned int i = 0, j = 0;
float current_duty;
char PWM_flag = 0;
char Example_State = 0;
char lab1[10] = {0};
char txt[10];

/*******************************************************************************
* Function ExtInt() iv IVT_INT_EXTI15_10 ics ICS_AUTO
* ------------------------------------------------------------------------------
* Overview: External interrupt on Button press
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void ExtInt() iv IVT_INT_EXTI15_10 ics ICS_AUTO {
  EXTI_PR.B10 = 1;                                                              // clear flag
  Example_State++;                                                              // Incmrement Example_State
}
/*******************************************************************************
* Function Display_Init()
* ------------------------------------------------------------------------------
* Overview: Function initialize display
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void Display_Init(){
  TFT_Init_ILI9341_8bit(320, 240);
  TFT_Set_Pen(CL_WHITE, 1);
  TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
  TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);

  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_Black, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_GREEN, FO_HORIZONTAL);
  TFT_Write_Text("Button  G  click", 85, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyMx PRO v7 for STM32", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&Tahoma52x52_Bold, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Countdown", 30, 60);
}
/*******************************************************************************
* Function BL_Fade_In()
* ------------------------------------------------------------------------------
* Overview: Fade In effect on TFT display
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void BL_Fade_In(){
  current_duty = 1;
  for(j=0;j<256;j++){
    PWM_TIM1_Set_Duty(current_duty,  _PWM_NON_INVERTED, _PWM_CHANNEL1);
    current_duty = current_duty * PWM_step;
    Delay_ms(2);
  }
}
/*******************************************************************************
* Function BL_Fade_Out()
* ------------------------------------------------------------------------------
* Overview: Fade Out effect on TFT display
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void BL_Fade_Out(){
  for(j=0;j<256;j++){
    PWM_TIM1_Set_Duty(current_duty,  _PWM_NON_INVERTED, _PWM_CHANNEL1);
    current_duty = current_duty / PWM_step;
    Delay_ms(2);
  }
}
/*******************************************************************************
* Function Button_PWM()
* ------------------------------------------------------------------------------
* Overview: Function drives Button LED diode with PWM
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Button_PWM(){
  if(PWM_flag == 0){
    current_duty = current_duty * PWM_step;
    PWM_TIM2_Set_Duty(current_duty,  _PWM_NON_INVERTED, _PWM_CHANNEL1);         // set newly acquired duty ratio

    i++;

    if(i == 255){
      PWM_flag = 1;
     }

    Delay_ms(4);
   }

  if(PWM_flag == 1){
    current_duty = current_duty / PWM_step ;
    PWM_TIM2_Set_Duty(current_duty,  _PWM_NON_INVERTED, _PWM_CHANNEL1);         // set newly acquired duty ratio

    i--;

    if(i == 0){
      PWM_flag = 0;
     }

    Delay_ms(4);
   }
}
/*******************************************************************************
* Function updateLabel(char *newText, int x, int y, char* lbl)
* ------------------------------------------------------------------------------
* Overview: Function writes text on display and erase old text
* Input: Nothing
* Output: Updated text
*******************************************************************************/
void updateLabel(char *newText, int x, int y, char* lbl){
  TFT_Set_Font(&Tahoma52x52_Bold, CL_WHITE, FO_HORIZONTAL);                     // Color of background
  TFT_Write_Text(lbl, x, y);
  strcpy(lbl, newText);
  TFT_Set_Font(&Tahoma52x52_Bold, CL_GREEN, FO_HORIZONTAL);                       // Color of text
  TFT_Write_Text(lbl, x, y);
}
/*******************************************************************************
* Function Countdown()
* ------------------------------------------------------------------------------
* Overview: Function countdown from 10 to 0 with sound
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Countdown(){
  char j;
  for(j=10;j>0;j--){                                                            // Countdown from 10 to 1
   ByteToStr(j, txt);                                                           // Convert Countdown number to string
   BL_Fade_In();                                                                // Fade In effect on TFT display
   Sound_Play(880, 25);                                                         // Play sound at 880Hz for 25 miliseconds
   updateLabel(txt, 130, 130, lab1);                                            // Write text on TFT display
   delay_ms(200);
   BL_Fade_Out();                                                               // Fade Out effect on TFT display
  }
}
//******************************************************************************
// Main function
//******************************************************************************
void main() {
  PWM_flag = 0;                                                                 // Set PWM flag as 0

  GPIO_Digital_Input(&GPIOD_IDR, _GPIO_PINMASK_10);                             // Set PD10 as digital input
  
  RCC_APB2ENR.AFIOEN = 1;                                                       // Enable clock for alternate pin functions
  AFIO_EXTICR3 = 0x0300;                                                        // PD10 as External interrupt
  EXTI_RTSR = 0x00000400;                                                       // Set interrupt on Rising edge
  EXTI_IMR |= 0x00000400;                                                       // Set mask
  NVIC_IntEnable(IVT_INT_EXTI15_10);                                            // Enable External interrupt

  EnableInterrupts();                                                           // Enables the processor interrupt.
  
  Display_Init();                                                               // Initialize TFT display
  Sound_Init(&GPIOE_ODR, 14);                                                   // Initialize buzzer on PE14

  PWM_TIM1_Init(10000);                                                         // Initialize PWM_TIM1
  PWM_TIM2_Init(10000);                                                         // Initialize PWM_TIM2

  PWM_TIM1_Start(_PWM_CHANNEL1, &_GPIO_MODULE_TIM1_CH1_PE9);                    // Start PWM_TIM1
  PWM_TIM2_Start(_PWM_CHANNEL1, &_GPIO_MODULE_TIM2_CH1_PA0);                    // Start PWM_TIM2
  
  current_duty = 1;                                                             // Set current duty as 1
  
  while (1) {                                                                   // endless loop
    switch (Example_State) {                                                    // State machine
      case 0 : {
                Button_PWM();                                                   // Fade In and Fade Out effect on Button LED diode
               }; break;
      case 1 : {                                                                // In case that Button is pressed
                PWM_TIM2_Stop(_PWM_CHANNEL1);                                   // Stop Fade In and Fade Out effect on Button LED diode
                Example_State = 2;                                              // Go to next state
               }; break;
      case 2 : {
                Countdown();                                                    // Start countdown
                Example_State = 3;                                              // Go to next state
               }; break;
      case 3 : {
                TFT_Image(0, 0, Launched_bmp, 1);                               // Draw image on TFT display
                BL_Fade_In();                                                   // Fade In effect on TFT display
                Sound_Play(880, 1000);                                          // Play sound at 880Hz for 1 second
                Example_State = 4;                                              // Go to next state
                break;
               }
       case 4 : {                                                               // Wait for Button press
                break;
               }
       case 5 : {                                                               // In case that Button is pressed
                BL_Fade_Out();                                                  // Fade Out effect on TFT display
                Display_Init();                                                 // Initialize TFT display
                PWM_TIM2_Start(_PWM_CHANNEL1, &_GPIO_MODULE_TIM2_CH1_PA0);      // Start PWM_TIM2
                current_duty = 1;                                               // Set current duty as 1
                i = 0;                                                          // Set i as 0
                PWM_flag = 0;                                                   // set PWM_flag as 0
                Example_State = 0;                                              // set Example_State as 0
                break;
               }
    }
  }
}
/*******************************************************************************
* End of File
*******************************************************************************/